<?php
/**
 * @file
 * Contains \Drupal\accessibility_tool\Form\AccessibilityToolSettingsForm.
 */

namespace Drupal\accessibility_tool\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\link\LinkItemInterface;
use Drupal\Core\Entity\Element\EntityAutocomplete;

/**
 * Class AccessibilityToolSettingsForm.
 *
 * @package Drupal\accessibility_tool\Form
 */
class AccessibilityToolSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'accessibility_tool_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('accessibility_tool.settings');

    // Positioning.
    $form['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#options' => [
        'right' => $this->t('Right'),
        'left' => $this->t('Left'),
      ],
      '#empty_option' => $this->t('- None -'),
      '#default_value' => $config->get('accessibility_tool.position'),
    ];

    // Tool color.
    $form['tool_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Tool color'),
      '#default_value' => $config->get('accessibility_tool.tool_color'),
    ];

    // Contrast colors.
    $contrast_color_count = $min_contrast_color = 1;
    $max_contrast_color = 10;
    $form_state->set('min_contrast_color', $contrast_color_count);
    $form_state->set('max_contrast_color', $max_contrast_color);

    if (!empty($form_state->get('contrast_color_count'))) {
      $contrast_color_count = $form_state->get('contrast_color_count');
    }
    else {
      if ($config->get('accessibility_tool.contrast_color_count')) {
        $contrast_color_count = $config->get('accessibility_tool.contrast_color_count');
        $form_state->set('contrast_color_count', $contrast_color_count);
      }
      else {
        $form_state->set('contrast_color_count', $contrast_color_count);
      }
    }

    $form['contrast_color'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Contrast colors'),
      '#attributes' => [],
      '#prefix' => '<div id="contrast-row-wrapper">',
      '#suffix' => '</div>',
    ];

    for ($i = 1; $i <= $contrast_color_count; $i++) {
      $form['contrast_color']['color_' . $i] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Contrast @num', ['@num' => $i]),
        '#attributes' => [],
        'color_background_' . $i => [
          '#type' => 'color',
          '#title' => $this->t('Background color'),
          '#default_value' => $config->get(
            'accessibility_tool.color_background_' . $i
          ),
        ],
        'color_foreground_' . $i => [
          '#type' => 'color',
          '#title' => $this->t('Foreground color'),
          '#default_value' => $config->get(
            'accessibility_tool.color_foreground_' . $i
          ),
        ],
      ];
    }

    if ($contrast_color_count < $max_contrast_color) {
      $form['contrast_color']['add_contrast'] = [
        '#type' => 'submit',
        '#value' => t('Add contrast'),
        '#submit' => ['::addContrastSubmit'],
      ];
    }
    if ($contrast_color_count > $min_contrast_color) {
      $form['contrast_color']['remove_contrast'] = [
        '#type' => 'submit',
        '#value' => t('Remove contrast'),
        '#submit' => ['::removeContrastSubmit'],
      ];
    }

    // Selectors.
    $form['selectors'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Added selectors'),
      '#description' => $this->t('Add selectors that will change the selected element to the background and foreground colors e.g. ".class div". This will be addition to the "at-contrast". Add each selector on a new line.'),
      '#default_value' => $config->get('accessibility_tool.selectors'),
    ];
    $form['alt_selectors'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Added alterative selectors'),
      '#description' => $this->t('Add alterative selectors that will change the selected element to the opposite background and foreground colors e.g. ".class div". This will be addition to the "at-alt-contrast" class. Add each selector on a new line.'),
      '#default_value' => $config->get('accessibility_tool.alt_selectors'),
    ];

    // Help url.
    $form['help_link'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Help link'),
      '#attributes' => [],
    ];

    $form['help_link']['help_link_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Help title'),
      '#default_value' => $config->get('accessibility_tool.help_link_title'),
    ];

    $default_uri = $config->get('accessibility_tool.help_link_url');
    if (!empty($default_uri)) {
      $default_uri = static::getUriAsDisplayableString($default_uri);
    }

    $form['help_link']['help_link_url'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Help URL'),
      '#default_value' => $default_uri,
      '#target_type' => 'node',
      '#attributes' => [
        'data-autocomplete-first-character-blacklist' => '/#?',
      ],
      '#process_default_value' => FALSE,
      '#element_validate' => [[$this, 'validateUriElement']],
      '#link_type' => LinkItemInterface::LINK_GENERIC,
    ];

    $form_state->setCached(FALSE);

    return $form;
  }

  /**
   * Submit handler for the "add-one-more" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public function addContrastSubmit(array &$form, FormStateInterface $form_state) {
    $max_contrast_color = $form_state->get('max_contrast_color');
    $contrast_color_count = $form_state->get('contrast_color_count');

    if ($max_contrast_color > $contrast_color_count) {
      $contrast_color_count++;
      $form_state->set('contrast_color_count', $contrast_color_count);
    }

    $form_state->setRebuild();
  }

  /**
   * Submit handler for the "add-one-more" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public function removeContrastSubmit(array &$form, FormStateInterface $form_state) {
    $min_contrast_color = $form_state->get('min_contrast_color');
    $contrast_color_count = $form_state->get('contrast_color_count');

    if ($min_contrast_color < $contrast_color_count) {
      $contrast_color_count--;
      $form_state->set('contrast_color_count', $contrast_color_count);
    }

    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('accessibility_tool.settings');
    $contrast_color_count = $form_state->get('contrast_color_count');

    $config->set(
      'accessibility_tool.position',
      $form_state->getValue('position'));

    $config->set(
      'accessibility_tool.tool_color',
      $form_state->getValue('tool_color'));

    $config->set(
      'accessibility_tool.contrast_color_count',
      $contrast_color_count);

    for ($i = 1;
         $i <= $config->get('accessibility_tool.contrast_color_count');
         $i++) {

      $config->set(
        'accessibility_tool.color_background_' . $i,
        $form_state->getValue('color_background_' . $i));
      $config->set(
        'accessibility_tool.color_foreground_' . $i,
        $form_state->getValue('color_foreground_' . $i)
      );
    }

    $config->set('accessibility_tool.selectors',
      $form_state->getValue('selectors'));
    $config->set('accessibility_tool.alt_selectors',
      $form_state->getValue('alt_selectors'));

    $config->set('accessibility_tool.help_link_url',
      $form_state->getValue('help_link_url'));
    $config->set('accessibility_tool.help_link_title',
      $form_state->getValue('help_link_title'));

    $config->save();

    // Clear all caches.
    drupal_flush_all_caches();

    return parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'accessibility_tool.settings',
    ];
  }

  /**
   * Form element validation handler for the 'uri' element.
   *
   * Disallows saving inaccessible or untrusted URLs.
   */
  public static function validateUriElement($element, FormStateInterface $form_state, $form) {
    $uri = static::getUserEnteredStringAsUri($element['#value']);
    $form_state->setValueForElement($element, $uri);

    // If getUserEnteredStringAsUri() mapped the entered value to a 'internal:'
    // URI , ensure the raw value begins with '/', '?' or '#'.
    if (parse_url($uri, PHP_URL_SCHEME) === 'internal' && !in_array($element['#value'][0], ['/', '?', '#'], TRUE) && substr($element['#value'], 0, 7) !== '<front>') {
      $form_state->setError($element, t('Manually entered paths should start with /, ? or #.'));
      return;
    }
  }

  /**
   * Gets the user-entered string as a URI.
   *
   * @param string $string
   *   The user-entered string.
   *
   * @return string
   *   The URI, if a non-empty $uri was passed.
   */
  protected static function getUserEnteredStringAsUri($string) {
    // By default, assume the entered string is an URI.
    $uri = trim($string);

    // Detect entity autocomplete string, map to 'entity:' URI.
    $entity_id = EntityAutocomplete::extractEntityIdFromAutocompleteInput($string);
    if ($entity_id !== NULL) {
      $uri = 'entity:node/' . $entity_id;
    }
    // Detect a schemeless string, map to 'internal:' URI.
    elseif (!empty($string) && parse_url($string, PHP_URL_SCHEME) === NULL) {

      if (strpos($string, '<front>') === 0) {
        $string = '/' . substr($string, strlen('<front>'));
      }
      $uri = 'internal:' . $string;
    }

    return $uri;
  }

  /**
   * Gets the URI without the 'internal:' or 'entity:' scheme.
   *
   * @param $uri
   *   The URI to get the displayable string for.
   *
   * @return string
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected static function getUriAsDisplayableString($uri) {
    $scheme = parse_url($uri, PHP_URL_SCHEME);

    // By default, the displayable string is the URI.
    $displayable_string = $uri;

    // A different displayable string may be chosen in case of the 'internal:'
    // or 'entity:' built-in schemes.
    if ($scheme === 'internal') {
      $uri_reference = explode(':', $uri, 2)[1];

      $path = parse_url($uri, PHP_URL_PATH);
      if ($path === '/') {
        $uri_reference = '<front>' . substr($uri_reference, 1);
      }
      $displayable_string = $uri_reference;
    }
    elseif ($scheme === 'entity') {
      list($entity_type, $entity_id) = explode('/', substr($uri, 7), 2);

      // Show the 'entity:' URI as the entity autocomplete would.
      if ($entity_type == 'node' &&
        $entity = \Drupal::entityTypeManager()
          ->getStorage($entity_type)
          ->load($entity_id)) {
        $displayable_string = EntityAutocomplete::getEntityLabels([$entity]);
      }
    }

    return $displayable_string;
  }
}
